import { ObservableArray } from '@nativescript/core';
import { isReactive, isRef, toRaw, watch } from 'nativescript-vue';
export var OnPreUpdateType;
(function (OnPreUpdateType) {
    OnPreUpdateType[OnPreUpdateType["Add"] = 0] = "Add";
    OnPreUpdateType[OnPreUpdateType["Update"] = 1] = "Update";
    OnPreUpdateType[OnPreUpdateType["Delete"] = 2] = "Delete";
})(OnPreUpdateType || (OnPreUpdateType = {}));
//type preSync<T, J> = (items: T[], syncType: OnPreSycType) => J[];
const baseExcludeCompareFields = { startingSide: null, menuOpened: null };
/**
 * Reactive synchronization between a reactive array with ObservableArray.
 *
 * @param arrayWatchTarge
 * @param options
 */
export function useSyncObservableArray(arrayRef, options = { addRemoveByField: '' }) {
    const { initialDelay = 0, checkRemoved = true, pushAllInFirstSync = false, checkAdded = true, checkUpdates = true, excludeCompareFields = undefined, addRemoveByField /* , onPreSync = undefined */, onPreUpdate = undefined, onPushInitialData = undefined } = options;
    const excludeFields = {
        ...baseExcludeCompareFields,
        ...excludeCompareFields?.reduce((a, b) => {
            a[b] = null;
            return a;
        }, {}),
    };
    let firstSync = true;
    //TODO: runOnPreSync
    //let clearArray = runOnPreSync(onPreSync, getClearArray(arrayRef), OnPreSycType.Initial);
    let clearArray = getClearArray(arrayRef);
    if (onPreUpdate) {
        clearArray = clearArray.map((item, index) => {
            return runOnPreUpdate(onPreUpdate, item, index, OnPreUpdateType.Add);
        });
    }
    const observableArray = createAndPushInitialData(clearArray, initialDelay, onPushInitialData);
    if (options?.watchUpdates && (isReactive(arrayRef) || isRef(arrayRef))) {
        watch(arrayRef, () => sync(), { deep: true });
    }
    function sync(newArray) {
        const itemList = newArray ? getClearArray(newArray) : getClearArray(arrayRef);
        //console.time("TIME_[useSyncObservableArray.sync]");
        //TODO: runOnPreSync
        //const clearArray = newArray ? getClearArray(newArray) : getClearArray(arrayRef);
        //const itemList = runOnPreSync(onPreSync, clearArray,  OnPreSycType.Update);
        //console.log('Processing_[useSyncObservableArray.sync.itemList.length] ' + itemList.length);
        if (pushAllInFirstSync && firstSync) {
            firstSync = false;
            observableArray.push(...itemList);
            return;
        }
        if (checkRemoved) {
            const indexRemoved = [];
            observableArray.forEach((itemObservable, index) => {
                const findItem = addRemoveByField ? itemList.find((item) => item[addRemoveByField] === itemObservable[addRemoveByField]) : itemList.find((item) => isEqualObject(item, itemObservable, excludeFields));
                if (!findItem) {
                    indexRemoved.push(index);
                }
            });
            indexRemoved.forEach((index) => {
                runOnPreUpdate(onPreUpdate, itemList[index], index, OnPreUpdateType.Delete);
                observableArray.splice(index, 1);
            });
        }
        if (checkAdded) {
            const indexAdd = [];
            itemList.forEach((item, index) => {
                const findItem = addRemoveByField ? observableArray.find((itemObservable) => itemObservable[addRemoveByField] === item[addRemoveByField]) : observableArray.find((itemObservable) => isEqualObject(itemObservable, item, excludeFields));
                if (!findItem) {
                    indexAdd.push(index);
                }
            });
            indexAdd.forEach((index) => {
                itemList[index] = runOnPreUpdate(onPreUpdate, itemList[index], index, OnPreUpdateType.Add);
                observableArray.splice(index, 0, itemList[index]);
            });
        }
        if (checkUpdates) {
            itemList.forEach((item, index) => {
                const itemObservable = observableArray.getItem(index);
                if (!isEqualObject(itemObservable, item, excludeFields)) {
                    item = runOnPreUpdate(onPreUpdate, item, index, OnPreUpdateType.Add);
                    observableArray.setItem(index, item);
                }
            });
        }
        //console.timeEnd("TIME_[useSyncObservableArray.sync]");
    }
    return {
        sync,
        observableArray,
    };
}
function createAndPushInitialData(clearArray, initialDelay, onPushInitialData) {
    let observableArray = new ObservableArray([]);
    if (initialDelay != 0) {
        setTimeout(() => {
            observableArray.push(...clearArray);
            if (onPushInitialData) {
                onPushInitialData();
            }
        }, initialDelay);
    }
    else {
        observableArray.push(...clearArray);
        if (onPushInitialData) {
            onPushInitialData();
        }
    }
    return observableArray;
}
/* function runOnPreSync<T, J>(onPreSync: preSync<T, J>, items: T[], type: OnPreSycType): T[] | J[] {
  if (onPreSync) return onPreSync(items, type);
  return items;
} */
function runOnPreUpdate(onPreUpdated, item, index, type) {
    if (onPreUpdated)
        return onPreUpdated(item, index, type);
    return item;
}
function getClearArray(array) {
    return cloneObject(extractArray(array));
}
function extractArray(array) {
    return isRef(array) ? array.value : array;
}
function isEqualObject(a, b, excludeFields) {
    const aObject = { ...a, ...excludeFields };
    const bObject = { ...b, ...excludeFields };
    return JSON.stringify(aObject) === JSON.stringify(bObject);
}
function cloneObject(object) {
    return JSON.parse(JSON.stringify(toRaw(object)));
}
//# sourceMappingURL=index.js.map